//@ts-check

import JadefinIntegrity from '../../../JadefinIntegrity.js';

import Jadefin from "../../../Jadefin.js";
import JadefinMod from "../../../JadefinMod.js";
import JadefinModules from "../../../JadefinModules.js";
import JadefinUtils from "../../../JadefinUtils.js";

class FilterworksAnime4KFilter {
    constructor(name, key) {
        this.name = name;
        this.key = key;

        this.a4k = window["Anime4KJS"];
        if (!this.a4k) {
            throw new Error("Anime4KJS not loaded");
        }

        this.profile = this.a4k[key];
    }

    /**
     * @param {HTMLVideoElement} video
     */
    start(video) {
        if (this.upscaler) {
            throw new Error("Filter already in use");
        }

        const stream = JadefinUtils.currentPlayer.streamInfo.mediaSource.MediaStreams.find(x => x.Type == "Video");
        const fps = stream?.RealFrameRate || stream?.AverageFrameRate || 60;
        this.upscaler = new this.a4k.VideoUpscaler(fps, this.profile);

        this.video = video;
        this.canvas = document.createElement("canvas");

        this.upscaler.attachVideo(this.video, this.canvas);
        this.upscaler.start();

        return this.canvas;
    }

    stop() {
        this.upscaler?.stop();
        this.upscaler?.detachVideo();
        this.upscaler = null;

        this.canvas?.remove();
        this.video?.removeAttribute("data-filterworks-canvas");

        this.video = null;
    }
}

export default JadefinIntegrity("FilterworksAnime4KSrc", import.meta.url, () => new (class FilterworksAnime4KSrc extends JadefinMod {
    filters = {
    }

    constructor() {
        super();
    }

    async init(name, url) {
        await super.init(name, url);

        await new Promise((resolve, reject) => {
            const loader = document.createElement("script");
            document.head.appendChild(loader);

            loader.addEventListener("load", () => resolve(true));
            loader.addEventListener("error", (e) => reject(e));

            loader.defer = true;
            loader.src = this.getUrl(".dist.js");
        });

        this.filters = {
            lqSIMPLE: new FilterworksAnime4KFilter("Anime4K Simple", "ANIME4KJS_SIMPLE_M_2X"),
            lqA: new FilterworksAnime4KFilter("Anime4K Mode A", "ANIME4K_LOWEREND_MODE_A_FAST"),
            lqB: new FilterworksAnime4KFilter("Anime4K Mode B", "ANIME4K_LOWEREND_MODE_B_FAST"),
            lqC: new FilterworksAnime4KFilter("Anime4K Mode C", "ANIME4K_LOWEREND_MODE_C_FAST"),
            /*
            hqSIMPLE: new FilterworksAnime4KFilter("Anime4K HQ Simple", "ANIME4KJS_SIMPLE_L_2X"),
            hqA: new FilterworksAnime4KFilter("Anime4K HQ Mode A", "ANIME4K_HIGHEREND_MODE_A_FAST"),
            hqB: new FilterworksAnime4KFilter("Anime4K HQ Mode B", "ANIME4K_HIGHEREND_MODE_B_FAST"),
            hqC: new FilterworksAnime4KFilter("Anime4K HQ Mode C", "ANIME4K_HIGHEREND_MODE_C_FAST")
            */
        };

        const Filterworks = /** @type {import("../Filterworks.js").default} */ (Jadefin.getMod("Filterworks"));

        Filterworks.sources.push(this);

        this.log.i("Ready");
    }
})());
